#include "config.h"

#ifdef SYSTEM_ZLIB
#include <zlib.h>
#endif

#include <stdio.h>
#include "mswordview.h"

#include <unistd.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>



extern FILE *outputfile;
extern FILE *erroroutput;

/*
written by peter.brandstrom@ericsson.com

Copyright (C) 1998
    Peter Brandstrom
*/

/*
wmf files are often stored compressed with lz compression in word files

<quote from documentation>
They are normally stored in a compressed format using the LZ compression 
algorithm in the format used by GNU Zip deflate/inflate with a 32k window. 
The format is zlib format . The only metafile compression version number 
currently defined identifies this format and is analogous to the PNG 
compression type value in the PNG file format.
</quote>

Peter Brandstrom - D/Z <peter.brandstrom@ericsson.com> took at look at it
and said
<quote>
The compressed data begins with a zlib header which seems to be two
bytes containing the compression method, window size and check byte. The
zlib default is a 32k window.
</quote>
*/

extern int errno;

int decompress(FILE *inputfile,char *outputfile,U32 inlen,U32 outlen)
	{
#ifdef SYSTEM_ZLIB
	char *compr;
	char *uncompr;
	int err;
	uLong uncomprLen, comprLen;
	int temp;
	FILE *fp;
	char *input,*output;
	int out;
	int in;
	int i;
	static int p;

	if (inputfile == NULL)
		{
		fprintf(erroroutput,"danger, file to decompress is NULL\n");
		return(-1);
		}

	/*
	fseek(inputfile,0,SEEK_END);
	fprintf(stderr,"in is %d, out is %d, real len is %d\n",inlen,outlen,ftell(inputfile));
	fseek(inputfile,0,SEEK_SET);
	*/

	in = fileno(inputfile);

	input = mmap(0,inlen,PROT_READ|PROT_WRITE,MAP_SHARED,in,0);

	if (input == (char *)-1)
		{
		fprintf(erroroutput,"unable to mmap inputfile\n");
		return(-1);
		}

	out = creat(outputfile,S_IRUSR|S_IWUSR);
	if (out == -1)
		{
		fprintf(erroroutput,"unable to create %s\n",outputfile);
		munmap(input,inlen);
		exit(-1);
		}

	for (i=0;i<outlen;i++)
		if (-1 == write(out,"0",1))
			{
			fprintf(erroroutput,"unable to write is %s\n",outputfile);
			munmap(input,inlen);
			close(out);
			exit(-1);
			}

	close(out);
	out = open(outputfile,O_RDWR);
	if (out == -1)
		{
		fprintf(erroroutput,"unable to open %s\n",outputfile);
		munmap(input,inlen);
		close(out);
		exit(-1);
		}

	output = mmap(0,outlen,PROT_READ|PROT_WRITE,MAP_SHARED,out,0);

	if (output == (char *)-1)
		{
		fprintf(erroroutput,"map out failed\n");
		fprintf(erroroutput,"%s\n",strerror(errno));
		munmap(input,inlen);
		close(out);
		exit(-1);
		}

	/*
	z_verbose = 1;
	*/
	
	/* set the size of the file*/
	comprLen = inlen;

	/* Read in the file contents */
	compr = input;
	uncompr = output;
	if (compr == NULL) 
		{
		fprintf(erroroutput,"no mem to decompress wmf files\n");
		return(-1);
		}
	if (uncompr == NULL) 
		{
		fprintf(erroroutput,"no mem to decompress wmf files\n");
		return(-1);
		}
	
	uncomprLen = outlen;	/* This was the trick :( */
	
	err = uncompress(uncompr, &uncomprLen, compr, comprLen);

	munmap(input,inlen);
	munmap(output,outlen);
	close(out);

	if (err != Z_OK) 
		{
		fprintf(erroroutput, "error: %d\n", err); 
		return(-1); 
		} 
	
#endif
	return 0;
	}
